package com.load.config;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.DescribeInstancesRequest;
import com.aliyun.ecs20140526.models.DescribeInstancesResponse;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.load.domain.ConnectWeight;
import com.load.util.ECSTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 初始化连接权重
 *
 * @author YunFei·Du
 * @ClassName: InitConnectWeight
 * @Description: 初始化连接权重
 * @CreateTime: 2024/5/27 21:31
 */
@Component
@Slf4j
public class InitConnectWeight implements ApplicationRunner {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private RestTemplate restTemplate;

    @Override
    public void run(ApplicationArguments args) throws Exception {

        ArrayList<ConnectWeight> connectWeightList = new ArrayList<>();

        ArrayList<String> ipList = new ArrayList<>();

        Client client = null;
        //获取阿里云客户端
        try {
            client = ECSTool.createClient();
            DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest().setRegionId("cn-zhangjiakou");
            DescribeInstancesResponse describeInstancesResponse = client.describeInstancesWithOptions(describeInstancesRequest, new RuntimeOptions());
            List<List<String>> ipListList = describeInstancesResponse.getBody().instances.getInstance().stream()
                    .map(instance -> instance.publicIpAddress.ipAddress)
                    .collect(Collectors.toList());
            ipListList.forEach(strings -> strings.forEach(ipList::add));
        } catch (Exception e) {
            log.error("获取阿里云客户端或实例信息失败", e);
            throw new RuntimeException(e);
        }
        for (String ip : ipList) {
            try {
                processIp(ip, connectWeightList);
            } catch (Exception e) {
                log.error("处理IP [{}] 时出错", ip, e);
            }
        }
//         connectWeightList 的逻辑

        // 初始化权重求和变量
        Integer sum =0;
        // 遍历ConnectWeight列表，计算所有权重值的总和
        for (ConnectWeight connectWeight : connectWeightList) {
            sum = sum + connectWeight.getWeightValue();
        }
        // 初始化最大轮询次数为0
        int max = 0;
        for (ConnectWeight connectWeight : connectWeightList) {
            // 将权重值转换为百分比并向下取整，得到每一轮的轮询次数
            Integer result = BigDecimal.valueOf(connectWeight.getWeightValue() * 100).divide(BigDecimal.valueOf(sum), 0, RoundingMode.DOWN).intValue();
            // 如果计算出的轮询次数大于当前最大值，更新最大值
            if (result > max){
                max = result;
            }
            // 更新ConnectWeight对象的权重值为计算出的轮询次数
            connectWeight.setWeightValue(result);
            // 输出100次轮询次数
            log.error("100次轮询次数:{}",result);
        }

        ArrayList<String> weightIpList = new ArrayList<>();

        //轮询出现次数
        for (int i = 0; i <= max; i++) {
            for (ConnectWeight connectWeight : connectWeightList) {
                if (connectWeight.getWeightValue() > i){
                    weightIpList.add(connectWeight.getCarServerIp());
                }else if (connectWeight.getWeightValue() == max){
                    weightIpList.add(connectWeight.getCarServerIp());
                }
            }
        }

        //存入redis
        redisTemplate.delete("ipList");
        for (String ip : weightIpList) {
            redisTemplate.opsForList().rightPush("ipList",ip);
        }
    }

    private void processIp(String ip, List<ConnectWeight> connectWeightList) throws Exception {

        String username = "fluxmq";
        String password = "fluxmq";
        String loginUrl = "http://" + ip + ":8080/public/login";
        String getInfoUrl = "http://" + ip + ":8080/public/cluster";

        Map< String, Object > loginRequest = new HashMap<> ( );
        loginRequest.put ( "username", username );
        loginRequest.put ( "password", password );

        HttpHeaders headers = new HttpHeaders ( );
        headers.setContentType ( MediaType.APPLICATION_JSON );
        HttpEntity< Map< String, Object > > loginEntity = new HttpEntity<> ( loginRequest, headers );

        // 登录操作
        String loginResult = restTemplate.postForObject ( loginUrl, loginEntity, String.class );
        // 使用登录cookie获取集群信息
        HttpHeaders headersGetInfo = new HttpHeaders ( );
        headersGetInfo.setContentType ( MediaType.APPLICATION_JSON );
        headersGetInfo.setAccept ( Collections.singletonList ( MediaType.APPLICATION_JSON ) );
        headersGetInfo.set ( "Cookie", loginResult );
        HttpEntity< String > getInfoEntity = new HttpEntity<> ( headersGetInfo );

        ResponseEntity< String > responseInfo = restTemplate.exchange ( getInfoUrl, HttpMethod.GET, getInfoEntity, String.class );

        if (responseInfo.getStatusCodeValue ( ) == 200) {
            JSONArray jsonArray = JSON.parseArray ( responseInfo.getBody ( ) );
            if (jsonArray.size ( ) > 0) {
                JSONObject jsonObject = jsonArray.getJSONObject ( 0 );

                log.info ( "响应 :{}", jsonObject );
                Integer connectSize = jsonObject.getJSONObject("mqttInfo").getJSONObject("runtimes").getInteger("mqtt.connect");
                ConnectWeight connectWeight = new ConnectWeight ( ip, 100 - connectSize );

                log.info ( "IP: {}, 连接数: {}, 权重值: {}", ip, connectSize, connectWeight.getWeightValue ( ) );

                log.info ( "Full Response: {}", responseInfo.getBody ( ) );
                connectWeightList.add ( connectWeight );
            } else {
                // 处理得到的响应数据为空的情况
                log.warn ( "响应数据为空" );
            }
        } else {
            // 处理HTTP请求失败的情况
            int statusCode = responseInfo.getStatusCodeValue ( );
            String responseBody = responseInfo.getBody ( );
            log.error ( "Failed to fetch data from {}: Status code {} with response: {}", ip, statusCode, responseBody );
        }
    }


}
